try:
    from redis import Redis
except ImportError:
    Redis = None

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum
from ..utils import run_sync_async


class HealthCheckRedis(HealthCheckAbstract):
    def __init__(self, uri: str):
        self.__uri = uri

    @property
    def service(self) -> str:
        return "redis"

    @property
    def connection_uri(self) -> str | None:
        return self.__uri

    @property
    def tags(self) -> list[str]:
        return ["redis", "cache", "database"]

    @property
    def comments(self) -> list[str]:
        return []

    def check_health(self) -> HealthCheckStatusEnum:
        if Redis is None:
            raise ImportError("redis is not installed. Install it with `pip install fastapi-healthz[redis]`.")

        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY
        try:
            db = Redis.from_url(self.__uri, socket_connect_timeout=5, socket_timeout=5)

            status = run_sync_async(db.ping)
            if status:
                res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res



import asyncio
import concurrent.futures
from enum import Enum as BaseEnum, EnumMeta
from typing import Callable, Any


class MetaEnum(EnumMeta):
    """
    Enables the use of the `in` operator for enums.
    For example:
    if el not in Elements:
        raise ValueError("invalid element")
    """
    def __contains__(cls, item):
        try:
            cls(item)
        except ValueError:
            return False
        return True


class Enum(BaseEnum, metaclass=MetaEnum):
    def __str__(self):
        return self.value

    def __eq__(self, other):
        if isinstance(other, Enum):
            return self.value == other.value
        return self.value == other

    def __hash__(self):
        return hash(self.value)


def run_sync_async(callback: Callable[..., Any], *args, **kwargs) -> Any:
    if not asyncio.iscoroutinefunction(callback) and not asyncio.iscoroutine(callback):
        return callback(*args, **kwargs)

    coro = callback(*args, **kwargs)

    try:
        asyncio.get_running_loop()

        def run_async_in_thread():
            return asyncio.run(coro)

        with concurrent.futures.ThreadPoolExecutor() as executor:
            future = executor.submit(run_async_in_thread)
            return future.result()
    except RuntimeError:
        return asyncio.run(coro)



from datetime import timedelta
from typing import Any
from pydantic import BaseModel, Field, field_serializer

from .utils import Enum as BaseEnum


class HealthCheckStatusEnum(BaseEnum):
    HEALTHY = "healthy"
    UNHEALTHY = "unhealthy"


class HealthCheckEntityModel(BaseModel):
    service: str
    status: str = Field(default=str(HealthCheckStatusEnum.HEALTHY))
    time_taken: timedelta = Field(default=timedelta())
    tags: list[str] = Field(default_factory=list)
    comments: list[str] = Field(default_factory=list)

    @field_serializer("time_taken")
    def serialize_time_taken(self, time: timedelta) -> str:
        # Pydantic will now call this to serialize the timedelta field
        return str(time)


class HealthCheckModel(BaseModel):
    status: str = Field(default=str(HealthCheckStatusEnum.HEALTHY))
    total_time_taken: timedelta = Field(default=timedelta())
    entities: list[dict[str, Any]] = Field(default_factory=list)

    @field_serializer("total_time_taken")
    def serialize_total_time_taken(self, time: timedelta) -> str:
        # Pydantic will now call this to serialize the timedelta field
        return str(time)



from .abstract import HealthCheckAbstract
from .mongodb_check import HealthCheckMongoDb
from .rabbitmq_check import HealthCheckRabbitMQ
from .database_check import HealthCheckDatabase
from .redis_check import HealthCheckRedis
from .s3 import HealthCheckS3
from .uri_check import HealthCheckUri



try:
    import boto3
    from sqlalchemy.orm import sessionmaker
except ImportError:
    boto3 = None

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckS3(HealthCheckAbstract):
    def __init__(self, endpoint: str, access_key: str, secret_key: str, ssl: bool):
        self.__endpoint = endpoint
        self.__access_key = access_key
        self.__secret_key = secret_key
        self.__ssl = ssl

    @property
    def service(self) -> str:
        return "s3"

    @property
    def connection_uri(self) -> str | None:
        return self.__endpoint

    @property
    def tags(self) -> list[str]:
        return ["s3"]

    @property
    def comments(self) -> list[str]:
        return []

    def check_health(self) -> HealthCheckStatusEnum:
        if boto3 is None:
            raise ImportError("boto3 is not installed. Install it with `pip install fastapi-healthz[s3]`.")

        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        try:
            s3_client = boto3.client(
                "s3",
                endpoint_url=self.__endpoint,
                aws_access_key_id=self.__access_key,
                aws_secret_access_key=self.__secret_key,
                verify=self.__ssl,
            )
            s3_client.list_buckets()
            res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res
